==> iRRAM_2004_01.tar.gz: iRRAM_2004_01/obj/MPFR/mpfr_extension.cc <==
/*

mpfr_extension.cc -- extension to the GMP float functions for the iRRAM library
 
Copyright (C) 2002/2003 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "iRRAM_core.h"
#include "mpfr_extension.h"
#include "mpfr-impl.h"

DYADIC glob_arg;
DYADIC glob_res;

void mp_ext_start(mpfr_ptr r, mpfr_srcptr u,long p, long proc(long)){
  mpfr_set_prec(glob_arg.value,mpfr_get_prec(u));
  mpfr_set(glob_arg.value,u,__gmp_default_rounding_mode);
  iRRAM_exec(proc,p);
  mpfr_set_prec(r,mpfr_get_prec(glob_res.value));
  mpfr_set(r,glob_res.value,__gmp_default_rounding_mode);
}

long compute_test(long p) { glob_res=approx(sin(REAL(glob_arg)),p); }
void mpfr_ext_test (mpfr_ptr y, mpfr_srcptr u, long p, mp_rnd_t rnd_mode)
   {
    mpfr_t r;
    mpfr_init(r);
    long err=50;
    if (p< 0) err= p;
    do  {
       err=err-10;
       mp_ext_start(r,u,err,compute_test);
 mpfr_print_binary(r);printf(" %d %d %d\n",MPFR_EXP(r),err,MPFR_EXP(r)+err);
     } while (!  mpfr_can_round(r,MPFR_EXP(r)+err ,GMP_RNDN, rnd_mode, p ));
 
     mpfr_set_prec(y,p);  
     mpfr_set(y,r,rnd_mode);
     
     mpfr_clear(r);
   }  

long compute_exp(long p)
   { glob_res=approx(exp(REAL(glob_arg)),p); }
void mpfr_ext_exp (mpfr_ptr r, mpfr_srcptr u, long p)
   { mp_ext_start(r,u,p,compute_exp); }  

long compute_log(long p)
   { glob_res=approx(log(REAL(glob_arg)),p); }
void mpfr_ext_log (mpfr_ptr r, mpfr_srcptr u, long p)
   { mp_ext_start(r,u,p,compute_log); }  

long compute_sin(long p)
   { glob_res=approx(sin(REAL(glob_arg)),p); }
void mpfr_ext_sin (mpfr_ptr r, mpfr_srcptr u, long p)
   { mp_ext_start(r,u,p,compute_sin); }  

long compute_cos(long p)
   { glob_res=approx(cos(REAL(glob_arg)),p); }
void mpfr_ext_cos (mpfr_ptr r, mpfr_srcptr u, long p)
   { mp_ext_start(r,u,p,compute_cos); }  

long compute_tan(long p)
   { glob_res=approx(tan(REAL(glob_arg)),p); }
void mpfr_ext_tan (mpfr_ptr r, mpfr_srcptr u, long p)
   { mp_ext_start(r,u,p,compute_tan); }

==> iRRAM_2005_01.tar.gz: iRRAM_2005_01/obj/MPFR/mpfr_extension.cc <==
/*

mpfr_extension.cc -- extension to the GMP float functions for the iRRAM library
 
Copyright (C) 2002/2003 Norbert Mueller
 
This file is part of the iRRAM Library.
 
The iRRAM Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Library General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at your
option) any later version.
 
The iRRAM Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
License for more details.
 
You should have received a copy of the GNU Library General Public License
along with the iRRAM Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
MA 02111-1307, USA. 
*/

#include "iRRAM_core.h"
#include "mpfr_extension.h"
#include "mpfr-impl.h"

DYADIC glob_arg;
DYADIC glob_res;

void mp_ext_start(mpfr_ptr r, mpfr_srcptr u,int p, int proc(int)){
  mpfr_set_prec(glob_arg.value,mpfr_get_prec(u));
  mpfr_set(glob_arg.value,u,__gmp_default_rounding_mode);
  iRRAM_exec(proc,p);
  mpfr_set_prec(r,mpfr_get_prec(glob_res.value));
  mpfr_set(r,glob_res.value,__gmp_default_rounding_mode);
}

int compute_test(int p) { glob_res=approx(sin(REAL(glob_arg)),p); }
void mpfr_ext_test (mpfr_ptr y, mpfr_srcptr u, int p, mp_rnd_t rnd_mode)
   {
    mpfr_t r;
    mpfr_init(r);
    int err=50;
    if (p< 0) err= p;
    do  {
       err=err-10;
       mp_ext_start(r,u,err,compute_test);
 mpfr_print_binary(r);printf(" %d %d %d\n",MPFR_EXP(r),err,MPFR_EXP(r)+err);
     } while (!  mpfr_can_round(r,MPFR_EXP(r)+err ,GMP_RNDN, rnd_mode, p ));
 
     mpfr_set_prec(y,p);  
     mpfr_set(y,r,rnd_mode);
     
     mpfr_clear(r);
   }  

int compute_exp(int p)
   { glob_res=approx(exp(REAL(glob_arg)),p); }
void mpfr_ext_exp (mpfr_ptr r, mpfr_srcptr u, int p)
   { mp_ext_start(r,u,p,compute_exp); }  

int compute_log(int p)
   { glob_res=approx(log(REAL(glob_arg)),p); }
void mpfr_ext_log (mpfr_ptr r, mpfr_srcptr u, int p)
   { mp_ext_start(r,u,p,compute_log); }  

int compute_sin(int p)
   { glob_res=approx(sin(REAL(glob_arg)),p); }
void mpfr_ext_sin (mpfr_ptr r, mpfr_srcptr u, int p)
   { mp_ext_start(r,u,p,compute_sin); }  

int compute_cos(int p)
   { glob_res=approx(cos(REAL(glob_arg)),p); }
void mpfr_ext_cos (mpfr_ptr r, mpfr_srcptr u, int p)
   { mp_ext_start(r,u,p,compute_cos); }  

int compute_tan(int p)
   { glob_res=approx(tan(REAL(glob_arg)),p); }
void mpfr_ext_tan (mpfr_ptr r, mpfr_srcptr u, int p)
   { mp_ext_start(r,u,p,compute_tan); }
